from typing import Optional

import django.forms

import asset_definitions

__all__ = ("MediaDefiningView",)


class MediaDefiningView(asset_definitions.MediaDefiningClass):
    def get_media(self) -> asset_definitions.Media:
        media = super(MediaDefiningView, self).get_media()
        form_media = self._get_form_media()
        if form_media:
            media += form_media
        return media

    def _get_form_media(self) -> Optional[django.forms.Media]:
        if hasattr(self, "get_form"):
            return self.get_form().media



from typing import Optional

import django.forms
import django.forms.widgets
import django.utils.html
import django.utils.safestring

__all__ = (
    "Media",
    "MediaDefiningClass",
)


# noinspection PyProtectedMember
# noinspection PyMissingConstructor
class Media(django.forms.Media):
    def __init__(self, **kwargs):
        self._media = django.forms.Media(**kwargs)
        self._combined_with = []

    @property
    def _js(self):
        media = django.forms.Media(
            js=self._media._js,
        )
        for other_media in self._combined_with:
            media += django.forms.Media(js=other_media._js)
        return media._js

    @property
    def _css(self):
        media = django.forms.Media(
            css=self._media._css,
        )
        for other_media in self._combined_with:
            media += django.forms.Media(css=other_media._css)
        return media._css

    def render_js(self):
        return [
            django.utils.safestring.mark_safe(js)
            if js.lstrip().startswith("<script")
            else django.utils.html.format_html(
                '<script type="text/javascript" src="{}"></script>',
                self.absolute_path(js),
            )
            for js in self._js
        ]

    def __getitem__(self, name):
        if name in django.forms.widgets.MEDIA_TYPES:
            return Media(**{str(name): getattr(self, "_" + name)})
        raise KeyError('Unknown media type "%s"' % name)

    def __add__(self, other: django.forms.Media) -> "Media":
        media = Media(js=self._media._js, css=self._media._css)
        media._combined_with = self._combined_with + [other]
        return media


class MediaDefiningClass(object):
    @property
    def media(self):
        return self.get_media()

    def get_media(self) -> Media:
        return self._get_media_from_definition() or Media()

    def _get_media_from_definition(self) -> Optional[Media]:
        definition = getattr(self.__class__, "Media", None)
        if definition:
            return Media(media=definition)



import os
import re
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest

        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


def read(fname):
    path = os.path.join(os.path.dirname(__file__), fname)
    if sys.version < "3":
        return open(path).read()
    return open(path, encoding="utf-8").read()


README = read("README.rst")
CHANGES = read("CHANGES.rst")


version = ""

with open("src/asset_definitions/__init__.py") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

if not version:
    raise RuntimeError("Cannot find version information")


setup(
    name="django-asset-definitions",
    packages=find_packages("src"),
    package_dir={"": "src"},
    version=version,
    author="Andrey Fedoseev",
    author_email="andrey.fedoseev@gmail.com",
    url="https://github.com/andreyfedoseev/django-asset-definitions",
    description="django-asset-definitions allows to define collections of static files (JavaScript, CSS) in Python "
    "code and (re)use them in Django views and templates.",
    long_description="\n\n".join([README, CHANGES]),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords=[
        "django",
    ],
    python_requires=">=3.6",
    install_requires=[
        "Django>=2.0",
    ],
    tests_require=[
        "pytest",
        "pytest-django",
    ],
    cmdclass={"test": PyTest},
)
